Package: MTLR
Type: Package
Title: Survival Prediction with Multi-Task Logistic Regression
Version: 0.2.2
Authors@R: person(given = "Humza",
                  family = "Haider",
                  role = c("aut", "cre"),
                  email = "humzahaider0@gmail.com")
URL: https://github.com/haiderstats/MTLR
BugReports: https://github.com/haiderstats/MTLR/issues
Description: An implementation of Multi-Task Logistic Regression (MTLR) for R. 
  This package is based on the method proposed by Yu et al. (2011) which utilized MTLR for generating individual survival curves
  by learning feature weights which vary across time. This model was further extended to account for left and interval censored data.
License: GPL-2 | file LICENSE
Encoding: UTF-8
Imports: Rcpp (>= 0.12.18), survival (>= 3.0.0), rlang
Suggests: ggplot2 (>= 3.4.0), reshape2 (>= 1.4.3), testthat, vdiffr (>=
        0.3.0), covr, knitr, rmarkdown
Depends: R (>= 3.4.0)
LinkingTo: Rcpp, RcppArmadillo
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-05-13 22:22:08 UTC; Humza.Haider
Author: Humza Haider [aut, cre]
Maintainer: Humza Haider <humzahaider0@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-14 08:40:13 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-05-14 09:25:24 UTC; unix
Archs: MTLR.so.dSYM
