# Generated by roxygen2: do not edit by hand

S3method(plot,mtlr)
S3method(predict,mtlr)
S3method(print,mtlr)
export(create_folds)
export(mtlr)
export(mtlr_cv)
export(plotcurves)
importFrom(Rcpp,sourceCpp)
importFrom(rlang,.data)
useDynLib(MTLR)
