# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

mtlr_objVal <- function(params, yval, featureValue, C1, delta) {
    .Call('_MTLR_mtlr_objVal', PACKAGE = 'MTLR', params, yval, featureValue, C1, delta)
}

mtlr_grad <- function(params, yval, featureValue, C1, delta) {
    .Call('_MTLR_mtlr_grad', PACKAGE = 'MTLR', params, yval, featureValue, C1, delta)
}

mtlr_predict <- function(params, featureValue) {
    .Call('_MTLR_mtlr_predict', PACKAGE = 'MTLR', params, featureValue)
}

